/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.apilib.common.command;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.common.datatype.StringBasedValue;
import lombok.EqualsAndHashCode;
import lombok.extern.log4j.Log4j2;

@Log4j2
@EqualsAndHashCode(callSuper = true)
public class OzgCloudCommandStatus extends StringBasedValue {

	public static final OzgCloudCommandStatus PENDING = new OzgCloudCommandStatus("PENDING");
	public static final OzgCloudCommandStatus FINISHED = new OzgCloudCommandStatus("FINISHED");
	public static final OzgCloudCommandStatus ERROR = new OzgCloudCommandStatus("ERROR");
	public static final OzgCloudCommandStatus REVOKE_PENDING = new OzgCloudCommandStatus("REVOKE_PENDING");
	public static final OzgCloudCommandStatus REVOKED = new OzgCloudCommandStatus("REVOKED");

	private static final Set<OzgCloudCommandStatus> ALL_STATES = new HashSet<>(Set.of(PENDING, FINISHED, ERROR, REVOKE_PENDING, REVOKED));
	private static final Set<OzgCloudCommandStatus> FINAL_STATES = Set.of(FINISHED, ERROR, REVOKED);

	OzgCloudCommandStatus(String status) {
		super(status);
	}

	public static OzgCloudCommandStatus from(String statusString) {
		return ALL_STATES.stream()
				.filter(status -> StringUtils.equals(statusString, status.toString())).findAny()
				.orElseGet(() -> unknownStatus(statusString));
	}

	private static OzgCloudCommandStatus unknownStatus(String statusString) {
		LOG.warn("Unknown status string given: {}", statusString);
		var newStatus = new OzgCloudCommandStatus(statusString);
		ALL_STATES.add(newStatus);
		return newStatus;
	}

	public boolean isFinalState() {
		return FINAL_STATES.contains(this);
	}
}
