/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.apilib.vorgang;

import java.util.Collection;
import java.util.Optional;

import lombok.Builder;
import lombok.Getter;
import lombok.Singular;

@Builder
@Getter
public class OzgCloudVorgangQuery {
	@Builder.Default
	private final Optional<String> organisationsEinheitId = Optional.empty();
	@Builder.Default
	private final Optional<FormIdentification> form = Optional.empty();
	@Builder.Default
	private final Optional<String> status = Optional.empty();
	@Builder.Default
	private final Optional<String> aktenzeichen = Optional.empty();

	@Builder.Default
	public final Optional<Query> query = Optional.empty();

	@Builder
	@Getter
	public static class Query {
		private final LogicalOperator logicalOperator;
		@Singular
		private final Collection<Expression> expressions;
		private final Query nestedQuery;
	}

	@Builder
	@Getter
	public static class FormIdentification {
		private String formId;
		private String formEngineName;
	}

	public enum LogicalOperator {
		AND, OR
	}

	@Builder
	@Getter
	public static class Expression {
		private final String path;
		private final ComparsionOperator operator;
		private final Integer operandIntValue;
		private final Boolean operandBoolValue;
		private final String operandStringValue;

		public enum ComparsionOperator {
			GREATER_THEN, // >
			GREATER_THEN_OR_EQUAL_TO, // >=
			LESS_THEN, // <
			LESS_THEN_OR_EQUAL_TO, // <=
			EQUAL, // =
			UNEQUAL, // <>
			IS_NULL,
			IS_EMPTY,
			EXISTS,
			NOT_EXIST;
		}

	}
}