/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.apilib.common.command;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;

class OzgCloudCommandStatusTest {

	@Nested
	class From {

		@Test
		void shouldReturnSameInstance() {
			OzgCloudCommandStatus result = OzgCloudCommandStatus.from("ERROR");

			assertThat(OzgCloudCommandStatus.ERROR).isSameAs(result);
		}

		@ParameterizedTest
		@ValueSource(strings = { "PENDING", "FINISHED", "ERROR", "REVOKE_PENDING", "REVOKED" })
		void shouldReturnSameInstance(String status) {
			OzgCloudCommandStatus result = OzgCloudCommandStatus.from(status);

			assertThat(result).isSameAs(OzgCloudCommandStatus.from(status));
		}

		@Test
		void shouldAllowUnkownStatusString() {
			var result = OzgCloudCommandStatus.from("SOMETHING_NEW");

			assertThat(result).isNotNull();
		}

		@Test
		void shouldLearnNewStatus() {
			var result = OzgCloudCommandStatus.from("SOMETHING_NEW");

			assertThat(result).isSameAs(OzgCloudCommandStatus.from("SOMETHING_NEW"));
		}
	}

}
