/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.client.autoconfigure;

import static org.assertj.core.api.Assertions.*;

import java.util.Optional;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Import;
import org.springframework.test.annotation.DirtiesContext;

import de.ozgcloud.apilib.autoconfig.ApiLibTestApplication;
import de.ozgcloud.apilib.client_attribute.OzgCloudBooleanClientAttribute;
import de.ozgcloud.apilib.client_attribute.OzgCloudClientAttributeService;
import de.ozgcloud.apilib.client_attribute.OzgCloudDoubleClientAttribute;
import de.ozgcloud.apilib.client_attribute.OzgCloudLongClientAttribute;
import de.ozgcloud.apilib.client_attribute.OzgCloudStringClientAttribute;
import de.ozgcloud.apilib.client_attribute.grpc.OzgCloudClientAttributeGrpcService;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangId;

class OzgCloudClientAttributeServiceStarterITCase {

	@Nested
	@SpringBootTest(classes = ApiLibTestApplication.class)
	@DirtiesContext
	class TestWithoutOzgCloudClientAttributeService {

		@Autowired
		private Optional<OzgCloudClientAttributeService> attributeService;

		@Test
		void shouldStart() {
			assertThat(attributeService).isEmpty();
		}
	}

	@Nested
	@SpringBootTest(classes = ApiLibTestApplication.class, properties = {
		"ozgcloud.vorgang-manager.address=address"
	})
	class TestWithOzgCloudClientAttributeService {

		@Autowired
		private OzgCloudClientAttributeService attributeService;

		@Test
		void shouldStart() {
			assertThat(attributeService).isInstanceOf(OzgCloudClientAttributeGrpcService.class);
		}
	}

	@Nested
	@SpringBootTest(classes = ApiLibTestApplication.class)
	@Import(TestWithExistingOzgCloudClientAttributeService.Config.class)
	class TestWithExistingOzgCloudClientAttributeService {

		@Autowired
		private OzgCloudClientAttributeService attributeService;

		@Test
		void shouldStart() {
			assertThat(attributeService).isInstanceOf(TestClientAttributeService.class);
		}

		static class Config {

			@Bean
			OzgCloudClientAttributeService ozgCloudFileService() {
				return new TestClientAttributeService();
			}
		}

		static class TestClientAttributeService implements OzgCloudClientAttributeService {

			@Override
			public void setClientAttribute(final OzgCloudBooleanClientAttribute clientAttribute, final OzgCloudVorgangId vorgangId) {
				// empty
			}

			@Override
			public void setClientAttribute(final OzgCloudStringClientAttribute clientAttribute, final OzgCloudVorgangId vorgangId) {
				// empty
			}

			@Override
			public void setClientAttribute(final OzgCloudLongClientAttribute clientAttribute, final OzgCloudVorgangId vorgangId) {
				// empty
			}

			@Override
			public void setClientAttribute(final OzgCloudDoubleClientAttribute clientAttribute, final OzgCloudVorgangId vorgangId) {
				// empty
			}
		}
	}
}
